function build_network(net)

fig = findobj('Type', 'figure', 'Tag', 'bbn');
if not(isempty(fig))
    set(0, 'CurrentFigure', fig);
    clf(fig) %deletes all children of the current figure with visible handles
else
    fig = figure('Name','Bayesian Belief Network','Tag','bbn');
    screensize = get(groot, 'Screensize');
    fig.OuterPosition = [10, 50, screensize(3)*1.1, screensize(4)*1.1];
end

% Create nodes
nodes = net.getAllNodeIds();
for i = 1:numel(nodes)
    node = nodes(i);
    
    switch net.getNodeType(node)
        case 18
            node_type = 'chance';
            outcomes = net.getOutcomeIds(node);
            beliefs = net.getNodeValue(node);
            
        case 4
            node_type = 'equation';
            outcomes = {'one','two'};
    end
    
    node_rect = net.getNodePosition(node);
    
    % Create panel and set panel position
    p = uipanel(fig, 'Tag', char(node), 'Title', replace(char(node), '_', ' '), 'TitlePosition', 'centertop', 'Units', 'pixels', 'FontName','times','FontSize',...
    12);
    
    p.Position(3) = 220;
    p.Position(4) = 30*numel(outcomes);
    overwidth = p.Position(3) - node_rect.getWidth();
    overheight = p.Position(4) - node_rect.getHeight();
    p.Position(1) = node_rect.getX() - .5*overwidth;
    p.Position(2) = fig.Position(4) - node_rect.getY() - .5*overheight;
    
    switch node_type
        case 'chance'
            ax = axes(p, 'Tag', char(node));
            bar(ax, 1:numel(beliefs), beliefs, 'Horizontal', 'on')
            ax.XTick = [];
            for j = 1:numel(ax.YTick)
                ax.YTickLabels{j} = replace([char(outcomes(j)), ' ', num2str(beliefs(j)*100), '%'], '_', ' ');
            end
            ax.OuterPosition(3) = 1.1;
            axis(ax, 'tight')
            ax.XLim = [0 1];
            
        case 'equation'
            tb = uicontrol('Style', 'text', 'Units', 'normalized');
            tb.Parent = p;
            tb.String = num2str(net.getNodeValue(node));
            tb.Position = [0 0 1 .5];
    end
end

% Create edges
for i = 1:numel(nodes)
    node = nodes(i);
    node_childs = net.getChildIds(node);
    panel = findobj('Type', 'uipanel', 'Tag', char(node));
    panel.Units = 'normalized';

    xp = panel.Position(1) + .5*panel.Position(3);
    yp = panel.Position(2) + .5*panel.Position(4);
    
    for j = 1:numel(node_childs)
        node_child = node_childs(j);
        panel_child = findobj('Type', 'uipanel', 'Tag', char(node_child));
        panel_child.Units = 'normalized';
        
        xc = panel_child.Position(1) + .5*panel_child.Position(3);
        yc = panel_child.Position(2) + .5*panel_child.Position(4);
        
        xdiff = xc - xp;
        ydiff = yc - yp;
        
        th = atan(ydiff/xdiff);
        th_lim = atan(panel_child.Position(4)/panel_child.Position(3));
        th2 = atan2(ydiff, xdiff);

        if abs(th) < abs(th_lim)
            rc = abs(.5*panel_child.Position(3)/cos(th2));
        else
            rc = abs(.5*panel_child.Position(4)/sin(th2));
        end
        xc = xc - rc*cos(th2);
        yc = yc - rc*sin(th2);
        
        annotation('textarrow', [xp xc], [yp yc])
    end
end
end